/* 	$Id$	 */

/* Copyright (c) 2007 Arnaud Legrand, Bruno Donnassolo.
   All rights reserved.                                          */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef SIMIX_PRIVATE_H
#define SIMIX_PRIVATE_H

#include <stdio.h>
#include "simix/simix.h"
#include "surf/surf.h"
#include "xbt/fifo.h"
#include "xbt/swag.h"
#include "xbt/dict.h"
#include "xbt/context.h"
#include "xbt/config.h"

/******************************* Datatypes **********************************/


/********************************** Host ************************************/

typedef struct s_smx_simdata_host {
  void *host;			/* SURF modeling */
  xbt_swag_t process_list;
} s_smx_simdata_host_t;

/********************************* Simix Global ******************************/

typedef struct SIMIX_Global {
  xbt_fifo_t host;
  xbt_swag_t process_to_run;
  xbt_swag_t process_list;

  smx_process_t current_process;
  xbt_dict_t registered_functions;
	void* (*create_process_function) ();
	void* (*kill_process_function)();
} s_SIMIX_Global_t, *SIMIX_Global_t;

extern SIMIX_Global_t simix_global;

/******************************* Process *************************************/

typedef struct s_smx_simdata_process {
  smx_host_t host;                /* the host on which the process is running */
  xbt_context_t context;	        /* the context that executes the scheduler fonction */
  int blocked;
  int suspended;
  smx_mutex_t mutex;		/* mutex on which the process is blocked  */
  smx_cond_t cond;		/* cond on which the process is blocked  */
  int argc;                     /* arguments number if any */
  char **argv;                  /* arguments table if any */
} s_smx_simdata_process_t;

typedef struct process_arg {
  const char *name;
  smx_process_code_t code;
  void *data;
  char *hostname;
  int argc;
  char **argv;
  double kill_time;
} s_process_arg_t, *process_arg_t;

/********************************* Mutex and Conditional ****************************/

typedef struct s_smx_mutex {
	xbt_swag_t sleeping;			/* list of sleeping process */
	int using;

} s_smx_mutex_t;

typedef struct s_smx_cond {
	xbt_swag_t sleeping; 			/* list of sleeping process */
	smx_mutex_t  mutex;
	xbt_fifo_t actions;			/* list of actions */

} s_smx_cond_t;

/********************************* Action **************************************/

typedef struct s_smx_simdata_action {
  surf_action_t surf_action;	/* SURF modeling of computation  */
  
  smx_host_t source; 

} s_smx_simdata_action_t;



/******************************* Configuration support **********************************/

void simix_config_init(void); /* create the config set, call this before use! */
void simix_config_finalize(void); /* destroy the config set, call this at cleanup. */
extern int _simix_init_status; /* 0: beginning of time; 
                                1: pre-inited (cfg_set created); 
                                2: inited (running) */
extern xbt_cfg_t _simix_cfg_set;


#define CHECK_HOST()  xbt_assert0(surf_workstation_resource->extension_public-> \
				  get_state(SIMIX_host_self()->simdata->host)==SURF_CPU_ON,\
                                  "Host failed, you cannot call this function.")

smx_host_t __SIMIX_host_create(const char *name, void *workstation, void *data);
void __SIMIX_host_destroy(smx_host_t host);

void __SIMIX_cond_wait(smx_cond_t cond);

void __SIMIX_display_process_status(void);



#endif
